#!/usr/bin/perl

##################################################################
# Block handling routines for the txt2docbook converter
#
# (c) Thomas Weber 2002
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Please see the file 'LICENSE.txt' for detailed copyright description 
# and the disclaimer  
#
# $Id: blocks.pm,v 1.3 2003/05/10 12:35:03 tweber Exp $
##################################################################

use Exporter;

@ISA=qw(Exporter);

package AbstractBlock;
	sub new {
		my $class=shift;
		my $indents=shift;
		my $this={};
		bless $this,$class;	
		$this->{indent}=$indents;
		return $this;
	}
	
	
	sub begin {
		
	}
	
	sub end {
		my $this=shift;
		my $nextblock=shift;

		return 0 if (!$nextblock);
		if ($nextblock eq "ENDDOCUMENT") {
			return 1;	
		};
		return 0;
	}
	
package Document;
	@ISA=qw( AbstractBlock );
	
	sub begin {
		my $this=shift;
		output::startDocument();
		
	}
	
	sub end {
		my $this=shift;
		my $nextblock=shift;
		return 0 if ($nextblock ne "ENDDOCUMENT" ); # Document end not allowed when another block is available
		output::endDocument();
		return 1;
	}
	
package DocumentTitle;
	@ISA=qw( AbstractBlock );
	
	sub new {
		my $class=shift;
		my $title=shift;
		
		my $this={};
		bless $this,$class;	
		$this->{title}=$title;
		
		return $this;
	}
	sub end {
		my $this=shift;
		my $nextblock=shift;
		output::printTitle($this->{title},$this->{author});
		return 1;
	}

package Paragraph;
	@ISA=qw( AbstractBlock );
	
	sub begin {
		my $this=shift;
		my $text=shift;
		output::startParagraph($text);
		
	}
	
	sub end {
		my $nextblock=shift;
		output::endParagraph();
		return 1;
	}
	

package Section;
	@ISA=qw( AbstractBlock );
	
	sub new {
		my $class=shift;
		my $indents=shift;
		my $level=shift;
		my $title=shift;
		
		my $this={};
		bless $this,$class;	
		$this->{indent}=$indents;
		$this->{level}=$level;
		$this->{title}=$title;
		
		return $this;
	}
	
	sub begin {
		my $this=shift;
		output::startSection($this->{level},$this->{title});
	}
	
	sub end {
		my $this=shift;
		my $nextblock=shift;
		
		if (AbstractBlock::end($this,$nextblock)) {
			output::endSection();
			return 1;
		}
				
		if (ref($nextblock) eq "Section") {
			if ($nextblock->{level} <= $this->{level}) {
				output::endSection();
				return 1;	
			}
		}
		
		return 0;
		
	}


package List;
	@ISA=qw( AbstractBlock );
	
	sub begin {
		my $this=shift;
		
		output::startList($this->{sublist});		
	}
	
	sub end {
		my $this=shift;
		my $nextblock=shift;
		my $sublist=$this->{sublist};

		if (AbstractBlock::end($this,$nextblock)) {
			output::endList($this->{sublist});
			return 1;
		}
		
		if (ref($nextblock) ne "ListItem") {
			if (!$this->{indent} || ($nextblock->{indent} <= $this->{indent}) ) {
				output::endList($this->{sublist});
				return 1;
			}
		}
		
		return 0;
	
	}


package ListItem;
	@ISA=qw( AbstractBlock );
	
	sub begin {
		my $this=shift;
		my $text=shift;		
		output::startListItem($text);

	}
	
	sub end {
		my $nextblock=shift;

		if (ref($nexblock) eq "Paragraph") {
			return 0;	
		}
		output::endListItem();
		return 1;
	}

package VarList;
	@ISA=qw( AbstractBlock );
	
	sub begin {
		my $this=shift;
		
		output::startVarList($this->{sublist});		
	}
	
	sub end {
		my $this=shift;
		my $nextblock=shift;
		my $sublist=$this->{sublist};

		if (AbstractBlock::end($this,$nextblock)) {
			output::endVarList($this->{sublist});
			return 1;
		}
		
		if (ref($nextblock) ne "VarListItem") {
			if ($this->{indent} && ($nextblock->{indent} <= $this->{indent})) {
				output::endVarList($this->{sublist});
				return 1;
			}
		}
		
		return 0;
	
	}

package VarListItem;
	@ISA=qw( AbstractBlock );
	
	sub begin {
		my $this=shift;
		my $term=shift;
		my $text=shift;		
		output::startVarListItem($term,$text);

	}
	
	sub end {
		my $nextblock=shift;
		
		output::endVarListItem();
		return 1;
	}


return 1;
#:collapseFolds=1:
