#!/usr/bin/perl

package output;

##################################################################
# Handles the output of the txt2docbook converter.
#
# (c) Thomas Weber 2002
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Please see the file 'LICENSE.txt' for detailed copyright description 
# and the disclaimer  
#
# Version: $Id: output.pl,v 1.3 2003/05/10 12:34:19 tweber Exp $
##################################################################

# !!!! IMPORTANT !!!!
# You have to set a valid path to your docbook-DTD here. This may be either a URL or a local path.
# If you choose to omit the XML system identifier, you need to comment out both $SYSTEMIDENTIFIER and $IDENTIFIER.
$SYSTEMIDENTIFIER="docbook/docbookx.dtd";
$IDENTIFIER='<!DOCTYPE chapter PUBLIC "-//OASIS//DTD DocBook XML V4.1.2//EN" "'.$SYSTEMIDENTIFIER.'">';
# !!!!!!!!!!!!!!!!!!!


##########################################
# Print a line to the output
##########################################

sub plainOutput {
	$text=shift;
	print STDOUT $text;
}

##########################################
# Generates and outputs a tag with indentation
##########################################

sub tagOutput {
	my $text=shift;
	
	# get the actual active block and retrieve the current indentation level
	my $block=::getCurrentblock();
	my $indent=$block->{indent};
	my $indentation="\t";
	while ($indent--) {
		$indentation.="\t";	
	}
	
	# Output the indentations and the tag
	plainOutput "$indentation".$text."\n";
	
}

##########################################
# Takes care of abbreviations and outputs
# a line of text
##########################################

sub textOutput {
	$text=shift;
	chomp $text;
	$text=doFormat($text);
	plainOutput $text."\n";
}

###########################################
# Outputs a text line in pass-through mode
###########################################
sub PassthroughOutput {
	my $tagname=shift;
	my $text=shift;
	plainOutput $text;	
}

##########################################
# Expand inline formatings to tags
# (abbreviations)
##########################################
sub doFormat {
	my $text=shift;
	
	# mangle '!...!' -> <emphasis>
	$text=~ s/(^|\s)\!(\w.*?\w)\!(\.|\,|:|\s|$)/ <emphasis>$2<\/emphasis> /g;
	# mangle '_..._' -> <filename>
	$text=~ s/(^|\s)_(\w.*?\w)_(\.|\,|:|\s|$)/ <filename>$2<\/filename> /g;
	# expand urls to ulink tags
	$text=~ s{(^|\s)(\w+://\S+)(\s|$)}{ <ulink url="$2"/> }g;
	
	return $text;
}

######################################################################
######################################################################
# BLOCK HANDLERS :
######################################################################
######################################################################

sub startDocument {
	plainOutput '<?xml version="1.0"  encoding="utf-8" ?>'
	."\n".$IDENTIFIER.'
<chapter>
	';	
}

sub endDocument {
	tagOutput "\n".'</chapter>'."\n";
}

##
# Creates the articleinfo tags
##
sub printTitle {
	my $title=shift;
	my $author=shift;
	my $first,$last;

	plainOutput '	
	<title>'.$title.'</title>
	';
	plainOutput '
	';
}

sub startPassthrough {
	my $tagname=shift;
	my $tagline=shift;
	tagOutput $tagline;
}

sub endPassthrough {
	my $tagname=shift;
	my $tagline=shift;
	tagOutput $tagline;
}

sub startSection {
	my $level=shift;
	my $title=shift;
	
	tagOutput '<section>';
	tagOutput '<title>';
	textOutput $title;
	tagOutput '</title>';
		
}

sub endSection {
	tagOutput '</section>'."\n";
	
}

sub startParagraph {
	my $text=shift;
	tagOutput '<para>';
	textOutput $text;
}

sub endParagraph {
	tagOutput '</para>';
}

sub startList {
	my $issublist=shift;
	tagOutput '<listitem>' if $issublist;
	tagOutput '<itemizedlist>';
}

sub endList {
	my $issublist=shift;
	tagOutput '</itemizedlist>';
	tagOutput '</listitem>' if $issublist;
}

sub startListItem {
	my $text=shift;
	tagOutput '<listitem><para>';
	textOutput $text;
}

sub endListItem {
	tagOutput '</para></listitem>';
}

sub startVarList {
	my $issublist=shift;
	tagOutput '<varlistentry>' if $issublist;
	tagOutput '<variablelist>';
}

sub endVarList {
	my $issublist=shift;
	tagOutput '</variablelist>';
	tagOutput '</varlistentry>' if $issublist;
}

sub startVarListItem {
	my $term=shift;
	my $text=shift;
	tagOutput '<varlistentry>';
	tagOutput '<term>'.$term.'</term>';
	tagOutput '<listitem><para>';
	textOutput $text;
}

sub endVarListItem {
	tagOutput '</para></listitem></varlistentry>';
}

return 1;
# :collapseFolds=1:
