#!/usr/bin/perl

package output;

##################################################################
# Handles the output of the txt2docbook converter.
#
# (c) Thomas Weber 2002
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Please see the file 'LICENSE.txt' for detailed copyright description 
# and the disclaimer  
#
# Version: $Id: output_html.pl,v 1.3 2003/05/10 12:34:19 tweber Exp $
##################################################################


##########################################
# Print a line to the output
##########################################

sub plainOutput {
	$text=shift;
	print STDOUT $text;
}

##########################################
# Generates and outputs a tag with indentation
##########################################

sub tagOutput {
	my $text=shift;
	
	# get the actual active block and retrieve the current indentation level
	my $block=::getCurrentblock();
	my $indent=$block->{indent};
	my $indentation="\t";
	while ($indent--) {
		$indentation.="\t";	
	}
	
	# Output the indentations and the tag
	plainOutput "$indentation".$text."\n";
	
}

##########################################
# Takes care of abbreviations and outputs
# a line of text
##########################################

sub textOutput {
	$text=shift;
	chomp $text;
	$text=doFormat($text);
	plainOutput $text."\n";
}

###########################################
# Outputs a text line in pass-through mode
#
###########################################
sub PassthroughOutput {
	my $tagname=shift;
	my $text=shift;
	
	# Remove CDATA lines which are not known in plain html
	$text =~ s/<!\[CDATA\[//; 
	$text =~ s/\]\]>//; 

	# Convert < and > to the appropriate html entities
	$text =~ s/</&lt;/g;
	$text =~ s/>/&gt;/g;

	plainOutput $text;	
}


##########################################
# Expand inline formatings to tags
# (abbreviations)
##########################################
sub doFormat {
	my $text=shift;
	
	# mangle '!...!' -> <emphasis>
	$text=~ s/(^|\s)\!(\w.*?\w)\!(\.|\,|:|\s|$)/ <font class='emphasis'>$2<\/font> /g;
	# mangle '_..._' -> <filename>
	$text=~ s/(^|\s)_(\w.*?\w)_(\.|\,|:|\s|$)/ <font class='filename'>$2<\/font> /g;
	# expand urls to anchor tags
	$text=~ s{(^|\s)(\w+://\S+)(\s|$)}{ <a href="$2">$2</a> }g;
	
	return $text;
}

######################################################################
######################################################################
# BLOCK HANDLERS :
######################################################################
######################################################################

sub startDocument {
	my $title=shift;
	plainOutput '
<html>
	';	
}

sub endDocument {
	tagOutput "\n".'</body></html>'."\n";
}

sub printTitle {
	my $title=shift;
	my $author=shift;
	my $first,$last;

	plainOutput '	
	<head>
		<title>'.$title.'</title>
	</head>
	<body>
	';
	plainOutput '
	<h1 class="title">'.$title.'</h1> <p class="author">'.$author.'</p> <br><hr>';

}

sub startPassthrough {
	my $tagname=shift;
	my $tagline=shift;
	tagOutput '<pre class="programlisting">';
}

sub endPassthrough {
	my $tagname=shift;
	my $tagline=shift;
	tagOutput '</pre>';
}

sub startSection {
	my $level=shift;
	my $title=shift;
	
	tagOutput '<p>';
	tagOutput '<H'.$level.'>';
	textOutput $title;
	tagOutput '</H'.$level.'>';
			
}

sub endSection {
	tagOutput '</p>'."\n";
	
}

sub startParagraph {
	my $text=shift;
	tagOutput '<p>';
	textOutput $text;
}

sub endParagraph {
	tagOutput '</p>';
}

sub startList {
	my $issublist=shift;
	tagOutput '<ul>';
}

sub endList {
	my $issublist=shift;
	tagOutput '</ul>';
}

sub startListItem {
	my $text=shift;
	tagOutput '<li>';
	textOutput $text;
}

sub endListItem {
	tagOutput '</li>';
}

sub startVarList {
	my $issublist=shift;
	tagOutput '<ul>';
}

sub endVarList {
	my $issublist=shift;
	tagOutput '</ul>';
}

sub startVarListItem {
	my $term=shift;
	my $text=shift;
	tagOutput '<li>';
	tagOutput '<p><font class="varlistitem">'.$term.'</font></p>';
	tagOutput '<p>';
	textOutput $text;
}

sub endVarListItem {
	tagOutput '</p></li>';
}

return 1;
# :collapseFolds=1:
